@@ -55,9 +55,12 @@ class ProgramConverter:
             # node already, DOT lets us add more information about that node
             # so we can insist on turning that same node into a box-shape
             # and changing its font.
-            output_str += ' "{}" [label="{}", fontname="{}", shape=box]\n'.format(func.name,
-                                                                                  ProgramConverter.get_display_name(func, suppress_common_nodes),
-                                                                                  font_name)
+            output_str += (
+                ' "{}" [label="{}", fontname="{}", fontsize="10", ' + 
+                'shape=box, penwidth=0.5]\n').format(
+                               func.name,
+                               ProgramConverter.get_display_name(func, suppress_common_nodes),
+                               font_name)
             if func.is_common:
                 output_str += ' "{}" [fillcolor=lightgreen, style=filled]\n'.format(func.name)
 
